
# DO NOT EDIT THIS FILE -- AUTOGENERATED BY PANTS
# Target: src/mitol/common:mitol-django-common

from setuptools import setup

setup(**{
    'authors': [
        'MIT Office of Open Learning <mitx-devops@mit.edu>',
    ],
    'classifiers': [
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
    ],
    'description': 'MIT Open Learning django app extensions for common utilities',
    'install_requires': (
        'django-webpack-loader>=0.7.0',
        'django<4.0,>=2.2.12',
        'factory-boy~=3.2',
        'pytest==6.1.2',
        'pytz>=2020.4',
        'requests>=2.20.0',
        'setuptools',
    ),
    'license': 'BSD 3-Clause License',
    'name': 'mitol-django-common',
    'namespace_packages': (
        'mitol',
    ),
    'package_data': {
        'mitol.common': (
            'CHANGELOG.md',
            'README.md',
            'py.typed',
        ),
    },
    'packages': (
        'mitol',
        'mitol.common',
        'mitol.common.factories',
        'mitol.common.management',
        'mitol.common.management.commands',
        'mitol.common.settings',
        'mitol.common.templatetags',
        'mitol.common.utils',
    ),
    'version': '2.2.4',
    'zip_safe': True,
})
